@@ -46,6 +46,8 @@ type Fetcher interface {
 	// Close() cleans up connections and state - must be paired with NewClient calls.
 	// returns how many sockets have been used (Fastclient only)
 	Close() int
+	// GetIPAddress() returns the last ip address used by this client connection.
+	GetIPAddress() string
 }
 
 const (
@@ -61,7 +63,7 @@ var (
 	contentLengthHeader   = []byte("\r\ncontent-length:")
 	connectionCloseHeader = []byte("\r\nconnection: close")
 	chunkedHeader         = []byte("\r\nTransfer-Encoding: chunked")
-	rander                = NewSyncReader(rand.New(rand.NewSource(time.Now().UnixNano())))
+	rander                = NewSyncReader(rand.New(rand.NewSource(time.Now().UnixNano()))) // nolint: gosec // we want fast not crypto
 )
 
 // NewHTTPOptions creates and initialize a HTTPOptions object.
@@ -182,6 +184,7 @@ type HTTPOptions struct {
 	LogErrors        bool          // whether to log non 2xx code as they occur or not
 	ID               int           // id to use for logging (thread id when used as a runner)
 	SequentialWarmup bool          // whether to do http(s) runs warmup sequentially or in parallel (new default is //)
+	ConnReuseRange   [2]int        // range of max number of connection to reuse for each thread.
 }
 
 // ResetHeaders resets all the headers, including the User-Agent: one (and the Host: logical special header).
@@ -199,19 +202,31 @@ func (h *HTTPOptions) InitHeaders() {
 	// before command line option -H are parsed/set.
 }
 
-// PayloadString returns the payload as a string. If payload is null return empty string
+// PayloadUTF8 returns the payload as a string. If payload is null return empty string
 // This is only needed due to grpc ping proto. It takes string instead of byte array.
-func (h *HTTPOptions) PayloadString() string {
-	if len(h.Payload) == 0 {
+func (h *HTTPOptions) PayloadUTF8() string {
+	p := h.Payload
+	pl := len(p)
+	if pl == 0 {
 		return ""
 	}
-	return string(h.Payload)
+	// grpc doesn't like invalid utf-8 strings, get rid of them
+	res := strings.ToValidUTF8(string(p), "")
+	l := len([]byte(res))
+	if l < pl {
+		// but then keep the expected bytes length (though it'll compressed unlike the original)
+		pad := pl - l
+		res += strings.Repeat("X", pad)
+		log.Infof("Padded payload with %d extra Xs to make valid UTF-8 after filtering invalid sequences", pad)
+		log.Debugf("Payload now %d bytes", len(res))
+	}
+	return res
 }
 
 // ValidateAndAddBasicAuthentication validates user credentials and adds basic authentication to http header,
 // if user credentials are valid.
 func (h *HTTPOptions) ValidateAndAddBasicAuthentication(headers http.Header) error {
-	if len(h.UserCredentials) <= 0 {
+	if len(h.UserCredentials) == 0 {
 		return nil // user credential is not entered
 	}
 	s := strings.SplitN(h.UserCredentials, ":", 2)
@@ -263,6 +278,37 @@ func (h *HTTPOptions) AddAndValidateExtraHeader(hdr string) error {
 	return nil
 }
 
+func (h *HTTPOptions) ValidateAndSetConnectionReuseRange(inp string) error {
+	if inp == "" {
+		return nil
+	}
+
+	reuseRangeString := strings.Split(inp, ":")
+	if len(reuseRangeString) > 2 {
+		return fmt.Errorf("more than two integers were provided in the connection reuse range")
+	}
+	reuseRangeInt := make([]int, 2)
+	for i, input := range reuseRangeString {
+		val, err := strconv.Atoi(input)
+		if err != nil {
+			return fmt.Errorf("invalid value for connection reuse range, err: %w", err)
+		}
+		reuseRangeInt[i] = val
+	}
+
+	if len(reuseRangeString) == 1 {
+		h.ConnReuseRange = [2]int{reuseRangeInt[0], reuseRangeInt[0]}
+	} else {
+		if reuseRangeInt[0] < reuseRangeInt[1] {
+			h.ConnReuseRange = [2]int{reuseRangeInt[0], reuseRangeInt[1]}
+		} else {
+			h.ConnReuseRange = [2]int{reuseRangeInt[1], reuseRangeInt[0]}
+		}
+	}
+
+	return nil
+}
+
 // newHttpRequest makes a new http GET request for url with User-Agent.
 func newHTTPRequest(o *HTTPOptions) (*http.Request, error) {
 	method := o.Method()
@@ -270,10 +316,9 @@ func newHTTPRequest(o *HTTPOptions) (*http.Request, error) {
 	if method == fnet.POST {
 		body = bytes.NewReader(o.Payload)
 	}
-	// nolint: noctx // TODO fixme?
 	req, err := http.NewRequest(method, o.URL, body)
 	if err != nil {
-		log.Errf("Unable to make %s request for %s : %v", method, o.URL, err)
+		log.Errf("[%d] Unable to make %s request for %s : %v", o.ID, method, o.URL, err)
 		return nil, err
 	}
 	req.Header = o.GenerateHeaders()
@@ -285,9 +330,9 @@ func newHTTPRequest(o *HTTPOptions) (*http.Request, error) {
 	}
 	bytes, err := httputil.DumpRequestOut(req, false)
 	if err != nil {
-		log.Errf("Unable to dump request: %v", err)
+		log.Errf("[%d] Unable to dump request: %v", o.ID, err)
 	} else {
-		log.Debugf("For URL %s, sending:\n%s", o.URL, bytes)
+		log.Debugf("[%d] For URL %s, sending:\n%s", o.ID, o.URL, bytes)
 	}
 	return req, nil
 }
@@ -299,7 +344,7 @@ type Client struct {
 	url                  string
 	path                 string // original path of the request's url
 	rawQuery             string // original query params
-	body                 string // original body of the request
+	body                 []byte // original body of the request
 	req                  *http.Request
 	client               *http.Client
 	transport            *http.Transport
@@ -308,15 +353,16 @@ type Client struct {
 	bodyContainsUUID     bool // if body contains the "{uuid}" pattern (lowercase)
 	logErrors            bool
 	id                   int
+	socketCount          int
 }
 
 // Close cleans up any resources used by NewStdClient.
 func (c *Client) Close() int {
-	log.Debugf("Close() on %+v", c)
+	log.Debugf("[%d] Close() on %+v", c.id, c)
 	if c.req != nil {
 		if c.req.Body != nil {
 			if err := c.req.Body.Close(); err != nil {
-				log.Warnf("Error closing std client body: %v", err)
+				log.Warnf("[%d] Error closing std client body: %v", c.id, err)
 			}
 		}
 		c.req = nil
@@ -324,7 +370,7 @@ func (c *Client) Close() int {
 	if c.transport != nil {
 		c.transport.CloseIdleConnections()
 	}
-	return 0 // TODO: find a way to track std client socket usage.
+	return c.socketCount
 }
 
 // ChangeURL only for standard client, allows fetching a different URL.
@@ -353,15 +399,15 @@ func (c *Client) Fetch() (int, []byte, int) {
 		c.req.URL.RawQuery = rawQuery
 	}
 	if c.bodyContainsUUID {
-		body := c.body
+		body := string(c.body)
 		for strings.Contains(body, uuidToken) {
 			body = strings.Replace(body, uuidToken, generateUUID(), 1)
 		}
 		bodyBytes := []byte(body)
 		c.req.ContentLength = int64(len(bodyBytes))
 		c.req.Body = ioutil.NopCloser(bytes.NewReader(bodyBytes))
-	} else if c.body != "" {
-		c.req.Body = ioutil.NopCloser(bytes.NewReader([]byte(c.body)))
+	} else if len(c.body) > 0 {
+		c.req.Body = ioutil.NopCloser(bytes.NewReader(c.body))
 	}
 
 	resp, err := c.client.Do(c.req)
@@ -372,9 +418,9 @@ func (c *Client) Fetch() (int, []byte, int) {
 	var data []byte
 	if log.LogDebug() {
 		if data, err = httputil.DumpResponse(resp, false); err != nil {
-			log.Errf("Unable to dump response %v", err)
+			log.Errf("[%d] Unable to dump response %v", c.id, err)
 		} else {
-			log.Debugf("For URL %s, received:\n%s", c.url, data)
+			log.Debugf("[%d] For URL %s, received:\n%s", c.id, c.url, data)
 		}
 	}
 	data, err = ioutil.ReadAll(resp.Body)
@@ -396,6 +442,11 @@ func (c *Client) Fetch() (int, []byte, int) {
 	return code, data, 0
 }
 
+// GetIPAddress get the ip address that DNS resolves to when using stdClient.
+func (c *Client) GetIPAddress() string {
+	return c.req.RemoteAddr
+}
+
 // NewClient creates either a standard or fast client (depending on
 // the DisableFastClient flag).
 func NewClient(o *HTTPOptions) (Fetcher, error) {
@@ -415,6 +466,23 @@ func NewStdClient(o *HTTPOptions) (*Client, error) {
 	if req == nil {
 		return nil, err
 	}
+
+	client := Client{
+		url:                  o.URL,
+		path:                 req.URL.Path,
+		pathContainsUUID:     strings.Contains(req.URL.Path, uuidToken),
+		rawQuery:             req.URL.RawQuery,
+		rawQueryContainsUUID: strings.Contains(req.URL.RawQuery, uuidToken),
+		body:                 o.Payload,
+		bodyContainsUUID:     strings.Contains(string(o.Payload), uuidToken),
+		req:                  req,
+		client: &http.Client{
+			Timeout: o.HTTPReqTimeOut,
+		},
+		id:        o.ID,
+		logErrors: o.LogErrors,
+	}
+
 	tr := http.Transport{
 		MaxIdleConns:        o.NumConnections,
 		MaxIdleConnsPerHost: o.NumConnections,
@@ -426,35 +494,35 @@ func NewStdClient(o *HTTPOptions) (*Client, error) {
 			if o.Resolve != "" {
 				addr = o.Resolve + addr[strings.LastIndex(addr, ":"):]
 			}
-			return (&net.Dialer{
+			var conn net.Conn
+			conn, err = (&net.Dialer{
 				Timeout: o.HTTPReqTimeOut,
 			}).DialContext(ctx, network, addr)
+
+			if conn != nil {
+				newRemoteAddress := conn.RemoteAddr().String()
+				// No change when it wasn't set before (first time) and when the value isn't actually changing either.
+				if req.RemoteAddr != "" && newRemoteAddress != req.RemoteAddr {
+					log.Infof("[%d] Standard client IP address changed from %s to %s", client.id, req.RemoteAddr, newRemoteAddress)
+				}
+				req.RemoteAddr = newRemoteAddress
+				client.socketCount++
+			}
+
+			return conn, err
 		},
 		TLSHandshakeTimeout: o.HTTPReqTimeOut,
 	}
+
+	client.client.Transport = &tr
+	client.transport = &tr
+
 	if o.https {
 		tr.TLSClientConfig, err = o.TLSOptions.TLSClientConfig()
 		if err != nil {
 			return nil, err
 		}
 	}
-	client := Client{
-		url:                  o.URL,
-		path:                 req.URL.Path,
-		pathContainsUUID:     strings.Contains(req.URL.Path, uuidToken),
-		rawQuery:             req.URL.RawQuery,
-		rawQueryContainsUUID: strings.Contains(req.URL.RawQuery, uuidToken),
-		body:                 o.PayloadString(),
-		bodyContainsUUID:     strings.Contains(o.PayloadString(), uuidToken),
-		req:                  req,
-		client: &http.Client{
-			Timeout:   o.HTTPReqTimeOut,
-			Transport: &tr,
-		},
-		transport: &tr,
-		id:        o.ID,
-		logErrors: o.LogErrors,
-	}
 	if !o.FollowRedirects {
 		// Lets us see the raw response instead of auto following redirects.
 		client.client.CheckRedirect = func(req *http.Request, via []*http.Request) error {
@@ -509,14 +577,23 @@ type FastClient struct {
 	id           int
 	https        bool
 	tlsConfig    *tls.Config
+	// range of connection reuse threshold that current thread will choose from
+	connReuseRange [2]int
+	connReuse      int
+	reuseCount     int
+}
+
+// GetIPAddress get ip address that DNS resolved to when using fast client.
+func (c *FastClient) GetIPAddress() string {
+	return c.dest.String()
 }
 
 // Close cleans up any resources used by FastClient.
 func (c *FastClient) Close() int {
-	log.Debugf("Closing %p %s socket count %d", c, c.url, c.socketCount)
+	log.Debugf("[%d] Closing %p %s socket count %d", c.id, c, c.url, c.socketCount)
 	if c.socket != nil {
 		if err := c.socket.Close(); err != nil {
-			log.Warnf("Error closing fast client's socket: %v", err)
+			log.Warnf("[%d] Error closing fast client's socket: %v", c.id, err)
 		}
 		c.socket = nil
 	}
@@ -542,25 +619,33 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) {
 		uuidStrings = append(uuidStrings, uuidString)
 		urlString = strings.Replace(urlString, uuidToken, uuidString, 1)
 	}
-	payload := o.PayloadString()
+	payload := string(o.Payload)
 	for strings.Contains(payload, uuidToken) {
 		uuidString := generateUUID()
 		uuidStrings = append(uuidStrings, uuidString)
 		payload = strings.Replace(payload, uuidToken, uuidString, 1)
 	}
-	o.Payload = []byte(payload)
-
+	if len(uuidStrings) > 0 {
+		o.Payload = []byte(payload)
+	}
 	// Parse the url, extract components.
 	url, err := url.Parse(urlString)
 	if err != nil {
-		log.Errf("Bad url '%s' : %v", urlString, err)
+		log.Errf("[%d] Bad url %q : %v", o.ID, urlString, err)
 		return nil, err
 	}
+
+	// Randomly assign a max connection reuse threshold to this thread.
+	var connReuse int
+	if o.ConnReuseRange != [2]int{0, 0} {
+		connReuse = generateReuseThreshold(o.ConnReuseRange[0], o.ConnReuseRange[1])
+	}
+
 	// note: Host includes the port
 	bc := FastClient{
 		url: o.URL, host: url.Host, hostname: url.Hostname(), port: url.Port(),
 		http10: o.HTTP10, halfClose: o.AllowHalfClose, logErrors: o.LogErrors, id: o.ID,
-		https: o.https,
+		https: o.https, connReuseRange: o.ConnReuseRange, connReuse: connReuse,
 	}
 	if o.https {
 		bc.tlsConfig, err = o.TLSOptions.TLSClientConfig()
@@ -571,11 +656,11 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) {
 	bc.buffer = make([]byte, BufferSizeKb*1024)
 	if bc.port == "" {
 		bc.port = url.Scheme // ie http which turns into 80 later
-		log.LogVf("No port specified, using %s", bc.port)
+		log.LogVf("[%d] No port specified, using %s", bc.id, bc.port)
 	}
 	var addr net.Addr
 	if o.UnixDomainSocket != "" { // nolint: nestif
-		log.Infof("Using unix domain socket %v instead of %v %v", o.UnixDomainSocket, bc.hostname, bc.port)
+		log.Infof("[%d] Using unix domain socket %v instead of %v %v", bc.id, o.UnixDomainSocket, bc.hostname, bc.port)
 		uds := &net.UnixAddr{Name: o.UnixDomainSocket, Net: fnet.UnixDomainSocket}
 		addr = uds
 	} else {
@@ -600,7 +685,7 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) {
 		host = o.hostOverride
 	}
 	if bc.tlsConfig != nil {
-		bc.tlsConfig.ServerName = host
+		bc.tlsConfig.ServerName = bc.hostname // Shouldn't have a port #571
 	}
 	var buf bytes.Buffer
 	buf.WriteString(method + " " + url.RequestURI() + " HTTP/" + proto + "\r\n")
@@ -633,7 +718,7 @@ func NewFastClient(o *HTTPOptions) (Fetcher, error) {
 			bc.uuidMarkers = append(bc.uuidMarkers, []byte(uuidString))
 		}
 	}
-	log.Debugf("Created client:\n%+v\n%s", bc.dest, bc.req)
+	log.Debugf("[%d] Created client:\n%+v\n%s", bc.id, bc.dest, bc.req)
 	return &bc, nil
 }
 
@@ -647,14 +732,15 @@ func (c *FastClient) connect() net.Conn {
 	c.socketCount++
 	var socket net.Conn
 	var err error
+	d := &net.Dialer{Timeout: c.reqTimeout}
 	if c.https {
-		socket, err = tls.Dial(c.dest.Network(), c.dest.String(), c.tlsConfig)
+		socket, err = tls.DialWithDialer(d, c.dest.Network(), c.dest.String(), c.tlsConfig)
 		if err != nil {
 			log.Errf("[%d] Unable to TLS connect to %v : %v", c.id, c.dest, err)
 			return nil
 		}
 	} else {
-		socket, err = net.Dial(c.dest.Network(), c.dest.String())
+		socket, err = d.Dial(c.dest.Network(), c.dest.String())
 		if err != nil {
 			log.Errf("[%d] Unable to connect to %v : %v", c.id, c.dest, err)
 			return nil
@@ -679,17 +765,25 @@ func (c *FastClient) Fetch() (int, []byte, int) {
 	c.headerLen = 0
 	// Connect or reuse existing socket:
 	conn := c.socket
-	reuse := (conn != nil)
-	if !reuse {
+	canReuse := conn != nil
+	if c.reachedReuseThreshold() {
+		c.connReuse = generateReuseThreshold(c.connReuseRange[0], c.connReuseRange[1])
+		log.LogVf("[%d] Thread reach the threshold for max connection canReuse of %d, force create new connection",
+			c.id, c.connReuse)
+	}
+
+	if conn == nil {
 		conn = c.connect()
+		c.reuseCount = 1
 		if conn == nil {
 			return c.returnRes()
 		}
 	} else {
-		log.Debugf("Reusing socket %v", conn)
+		c.reuseCount++
+		log.Debugf("[%d] Reusing socket %v", c.id, c.dest)
 	}
 	c.socket = nil // because of error returns and single retry
-	conErr := conn.SetReadDeadline(time.Now().Add(c.reqTimeout))
+	conErr := conn.SetDeadline(time.Now().Add(c.reqTimeout))
 	// Send the request:
 	req := c.req
 	if len(c.uuidMarkers) > 0 {
@@ -699,7 +793,7 @@ func (c *FastClient) Fetch() (int, []byte, int) {
 	}
 	n, err := conn.Write(req)
 	if err != nil || conErr != nil {
-		if reuse {
+		if canReuse {
 			// it's ok for the (idle) socket to die once, auto reconnect:
 			log.Infof("[%d] Closing dead socket %v (%v)", c.id, c.dest, err)
 			conn.Close()
@@ -720,13 +814,13 @@ func (c *FastClient) Fetch() (int, []byte, int) {
 				log.Errf("[%d] Unable to close write to %v : %v", c.id, c.dest, err)
 				return c.returnRes()
 			} // else:
-			log.Debugf("Half closed ok after sending request %v %v", conn, c.dest)
+			log.Debugf("[%d] Half closed ok after sending request %v", c.id, c.dest)
 		} else {
 			log.Warnf("[%d] Unable to close write non tcp connection %v", c.id, c.dest)
 		}
 	}
 	// Read the response:
-	c.readResponse(conn, reuse)
+	c.readResponse(conn, canReuse)
 	if c.code == RetryOnce {
 		// Special "eof on reused socket" code
 		return c.Fetch() // recurse once
@@ -779,8 +873,8 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 			}
 			c.size += n
 			if log.LogDebug() {
-				log.Debugf("Read ok %d total %d so far (-%d headers = %d data) %s",
-					n, c.size, c.headerLen, c.size-c.headerLen, DebugSummary(c.buffer[c.size-n:c.size], 256))
+				log.Debugf("[%d] Read ok %d total %d so far (-%d headers = %d data) %s",
+					c.id, n, c.size, c.headerLen, c.size-c.headerLen, DebugSummary(c.buffer[c.size-n:c.size], 256))
 			}
 		}
 		skipRead = false
@@ -797,8 +891,8 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 				break
 			}
 			if log.LogDebug() {
-				log.Debugf("Code %d, looking for end of headers at %d / %d, last CRLF %d",
-					c.code, endofHeadersStart, c.size, c.headerLen)
+				log.Debugf("[%d] Code %d, looking for end of headers at %d / %d, last CRLF %d",
+					c.id, c.code, endofHeadersStart, c.size, c.headerLen)
 			}
 			// TODO: keep track of list of newlines to efficiently search headers only there
 			idx := endofHeadersStart
@@ -818,7 +912,7 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 				// We have headers !
 				c.headerLen += 4 // we use this and not endofHeadersStart so http/1.0 does return 0 and not the optimization for search start
 				if log.LogDebug() {
-					log.Debugf("headers are %d: %s", c.headerLen, c.buffer[:idx])
+					log.Debugf("[%d] headers are %d: %q", c.id, c.headerLen, c.buffer[:idx])
 				}
 				// Find the content length or chunked mode
 				if keepAlive {
@@ -834,7 +928,7 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 						}
 						max = c.headerLen + contentLength
 						if log.LogDebug() { // somehow without the if we spend 400ms/10s in LogV (!)
-							log.Debugf("found content length %d", contentLength)
+							log.Debugf("[%d] found content length %d", c.id, contentLength)
 						}
 					} else {
 						// Chunked mode (or err/missing):
@@ -844,17 +938,17 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 							dataStart, contentLength = ParseChunkSize(c.buffer[c.headerLen:c.size])
 							if contentLength == -1 {
 								// chunk length not available yet
-								log.LogVf("chunk mode but no first chunk length yet, reading more")
+								log.LogVf("[%d] chunk mode but no first chunk length yet, reading more", c.id)
 								max = c.headerLen
 								continue
 							}
 							max = c.headerLen + dataStart + contentLength + 2 // extra CR LF
-							log.Debugf("chunk-length is %d (%s) setting max to %d",
-								contentLength, c.buffer[c.headerLen:c.headerLen+dataStart-2],
+							log.Debugf("[%d] chunk-length is %d (%s) setting max to %d",
+								c.id, contentLength, c.buffer[c.headerLen:c.headerLen+dataStart-2],
 								max)
 						} else {
 							if log.LogVerbose() {
-								log.LogVf("Warning: content-length missing in %s", string(c.buffer[:c.headerLen]))
+								log.LogVf("[%d] Warning: content-length missing in %q", c.id, string(c.buffer[:c.headerLen]))
 							} else {
 								log.Warnf("[%d] Warning: content-length missing (%d bytes headers)", c.id, c.headerLen)
 							}
@@ -880,10 +974,10 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		} // end of big if parse header
 		if c.size >= max {
 			if !keepAlive {
-				log.Errf("More data is available but stopping after %d, increase -httpbufferkb", max)
+				log.Errf("[%d] More data is available but stopping after %d, increase -httpbufferkb", c.id, max)
 			}
 			if !parsedHeaders && c.parseHeaders {
-				log.Errf("Buffer too small (%d) to even finish reading headers, increase -httpbufferkb to get all the data", max)
+				log.Errf("[%d] Buffer too small (%d) to even finish reading headers, increase -httpbufferkb to get all the data", c.id, max)
 				keepAlive = false
 			}
 			if chunkedMode {
@@ -891,24 +985,25 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 				dataStart, nextChunkLen := ParseChunkSize(c.buffer[max:c.size])
 				if nextChunkLen == -1 {
 					if c.size == max {
-						log.Debugf("Couldn't find next chunk size, reading more %d %d", max, c.size)
+						log.Debugf("[%d] Couldn't find next chunk size, reading more %d %d", c.id, max, c.size)
 					} else {
-						log.Infof("Partial chunk size (%s), reading more %d %d", DebugSummary(c.buffer[max:c.size], 20), max, c.size)
+						log.Infof("[%d] Partial chunk size (%s), reading more %d %d", c.id, DebugSummary(c.buffer[max:c.size], 20), max, c.size)
 					}
 					continue
 				} else if nextChunkLen == 0 {
-					log.Debugf("Found last chunk %d %d", max+dataStart, c.size)
+					log.Debugf("[%d] Found last chunk %d %d", c.id, max+dataStart, c.size)
 					if c.size != max+dataStart+2 || string(c.buffer[c.size-2:c.size]) != "\r\n" {
-						log.Errf("Unexpected mismatch at the end sz=%d expected %d; end of buffer %q", c.size, max+dataStart+2, c.buffer[max:c.size])
+						log.Errf("[%d] Unexpected mismatch at the end sz=%d expected %d; end of buffer %q",
+							c.id, c.size, max+dataStart+2, c.buffer[max:c.size])
 					}
 				} else {
 					max += dataStart + nextChunkLen + 2 // extra CR LF
-					log.Debugf("One more chunk %d -> new max %d", nextChunkLen, max)
+					log.Debugf("[%d] One more chunk %d -> new max %d", c.id, nextChunkLen, max)
 					if max > len(c.buffer) {
-						log.Errf("Buffer too small for %d data", max)
+						log.Errf("[%d] Buffer too small for %d data", c.id, max)
 					} else {
 						if max <= c.size {
-							log.Debugf("Enough data to reach next chunk, skipping a read")
+							log.Debugf("[%d] Enough data to reach next chunk, skipping a read", c.id)
 							skipRead = true
 						}
 						continue
@@ -919,19 +1014,39 @@ func (c *FastClient) readResponse(conn net.Conn, reusedSocket bool) {
 		}
 	} // end of big for loop
 	// Figure out whether to keep or close the socket:
-	if keepAlive && codeIsOK(c.code) {
+	if keepAlive && codeIsOK(c.code) && !c.reachedReuseThreshold() {
 		c.socket = conn // keep the open socket
 	} else {
 		if err := conn.Close(); err != nil {
 			log.Errf("[%d] Close error %v %d : %v", c.id, c.dest, c.size, err)
 		} else {
-			log.Debugf("Closed ok %v from %v after reading %d bytes", conn, c.dest, c.size)
+			log.Debugf("[%d] Closed ok from %v after reading %d bytes", c.id, c.dest, c.size)
 		}
 		// we cleared c.socket in caller already
 	}
 }
 
+// Check if current thread reached the connection reuse threshold.
+func (c *FastClient) reachedReuseThreshold() bool {
+	if c.connReuse != 0 && c.reuseCount >= c.connReuse {
+		log.LogVf("[%d] Thread reach the threshold for max connection reuse of %d, force create new connection",
+			c.id, c.connReuse)
+		return true
+	}
+
+	return false
+}
+
 func generateUUID() string {
 	// We use math random instead of crypto random generator due to performance.
 	return uuid.Must(uuid.NewRandomFromReader(rander)).String()
 }
+
+// Generate reuse threshold based on the min and max value in the flag.
+func generateReuseThreshold(min int, max int) int {
+	if min == max {
+		return min
+	}
+
+	return min + rand.Intn(max-min+1) // nolint: gosec // we want fast not crypto
+}
