@@ -27,6 +27,7 @@ import (
 	"strings"
 	"testing"
 	"time"
+	"unicode/utf8"
 
 	"fortio.org/fortio/fnet"
 	"fortio.org/fortio/log"
@@ -93,6 +94,18 @@ func TestNewHTTPRequest(t *testing.T) {
 	}
 }
 
+func TestPayloadUTF8(t *testing.T) {
+	opts := HTTPOptions{}
+	opts.Payload = fnet.GenerateRandomPayload(1024)
+	res := opts.PayloadUTF8()
+	if l := len(res); l != 1024 {
+		t.Errorf("Unexpected length %d", l)
+	}
+	if !utf8.ValidString(res) {
+		t.Errorf("PayloadUTF8() not returning valid utf-8 string")
+	}
+}
+
 func TestMultiInitAndEscape(t *testing.T) {
 	// 2 escaped already
 	o := NewHTTPOptions("localhost%3A8080/?delay=10ms:10,0.5s:15%25,0.25s:5")
@@ -617,7 +630,7 @@ func TestEchoBack(t *testing.T) {
 	v := url.Values{}
 	v.Add("foo", "bar")
 	url := fmt.Sprintf("http://localhost:%d/?delay=2s", a.Port) // trigger max delay
-	resp, err := http.PostForm(url, v)                          // nolint: noctx // it's just a test!
+	resp, err := http.PostForm(url, v)
 	if err != nil {
 		t.Fatalf("post form err %v", err)
 	}
@@ -676,7 +689,7 @@ func TestSmallBufferAndNoKeepAlive(t *testing.T) {
 	cli.Close()
 }
 
-func TestBadUrl(t *testing.T) {
+func TestBadUrlFastClient(t *testing.T) {
 	opts := NewHTTPOptions("not a valid url")
 	cli, err := NewFastClient(opts)
 	if cli != nil || err == nil {
@@ -691,6 +704,22 @@ func TestBadUrl(t *testing.T) {
 	}
 }
 
+func TestBadURLStdClient(t *testing.T) {
+	opts := NewHTTPOptions("not a valid url")
+	cli, err := NewStdClient(opts)
+	if cli != nil || err == nil {
+		t.Errorf("config1: got a client %v despite bogus url %s", cli, opts.URL)
+		cli.Close()
+	}
+
+	opts.URL = "http://doesnotexist.fortio.org"
+	cli, _ = NewStdClient(opts)
+	code, _, _ := cli.Fetch()
+	if code != -1 {
+		t.Errorf("config2: client can send request despite bogus url %s", opts.URL)
+	}
+}
+
 func TestDefaultPort(t *testing.T) {
 	// TODO: change back to fortio demo server once setup
 	url := "http://istio.io/" // shall imply port 80
@@ -1133,7 +1162,7 @@ func TestEchoHeaders(t *testing.T) {
 	// proper encoding
 	urls = append(urls, fmt.Sprintf("http://localhost:%d/echo?%s", a.Port, v.Encode()))
 	for _, url := range urls {
-		resp, err := http.Get(url) // nolint: noctx // it's just a test!
+		resp, err := http.Get(url)
 		if err != nil {
 			t.Fatalf("Failed get for %s : %v", url, err)
 		}
@@ -1414,7 +1443,7 @@ func TestValidateAndAddBasicAuthentication(t *testing.T) {
 		if err == nil && !test.isCredentialsValid {
 			t.Errorf("Error was not expected for %s", test.o.UserCredentials)
 		}
-		if test.isAuthHeaderAdded && len(h.Get("Authorization")) <= 0 {
+		if test.isAuthHeaderAdded && len(h.Get("Authorization")) == 0 {
 			t.Errorf("Authorization header was expected for %s credentials", test.o.UserCredentials)
 		}
 	}
