@@ -20,6 +20,7 @@ import (
 	"bytes"
 	"compress/gzip"
 	"fmt"
+	"math"
 	"net/http"
 	"os"
 	"path"
@@ -60,6 +61,10 @@ func TestHTTPRunner(t *testing.T) {
 	if res.SocketCount != res.RunnerResults.NumThreads {
 		t.Errorf("%d socket used, expected same as thread# %d", res.SocketCount, res.RunnerResults.NumThreads)
 	}
+	count := getIPUsageCount(res.IPCountMap)
+	if count != res.RunnerResults.NumThreads {
+		t.Errorf("Total IP usage count %d, expected same as thread %d", count, res.RunnerResults.NumThreads)
+	}
 	// Test raw client, should get warning about non init timeout:
 	rawOpts := HTTPOptions{
 		URL: opts.URL,
@@ -121,11 +126,8 @@ func testHTTPNotLeaking(t *testing.T, opts *HTTPRunnerOptions) {
 	if ngAfter > ngBefore2+8 {
 		t.Errorf("Goroutines after test %d, expected it to stay near %d", ngAfter, ngBefore2)
 	}
-	if !opts.DisableFastClient {
-		// only fast client so far has a socket count
-		if res.SocketCount != res.RunnerResults.NumThreads {
-			t.Errorf("%d socket used, expected same as thread# %d", res.SocketCount, res.RunnerResults.NumThreads)
-		}
+	if res.SocketCount != res.RunnerResults.NumThreads {
+		t.Errorf("%d socket used, expected same as thread# %d", res.SocketCount, res.RunnerResults.NumThreads)
 	}
 }
 
@@ -190,17 +192,16 @@ func TestHTTPRunnerClientRace(t *testing.T) {
 	}
 }
 
-func TestClosingAndSocketCount(t *testing.T) {
+func testClosingAndSocketCount(t *testing.T, o *HTTPRunnerOptions) {
 	mux, addr := DynamicHTTPServer(false)
 	mux.HandleFunc("/echo42/", EchoHandler)
 	URL := fmt.Sprintf("http://localhost:%d/echo42/?close=true", addr.Port)
-	opts := HTTPRunnerOptions{}
-	opts.Init(URL)
-	opts.QPS = 10
+	o.Init(URL)
+	o.QPS = 10
 	numReq := int64(50) // can't do too many without running out of fds on mac
-	opts.Exactly = numReq
-	opts.NumThreads = 5
-	res, err := RunHTTPTest(&opts)
+	o.Exactly = numReq
+	o.NumThreads = 5
+	res, err := RunHTTPTest(o)
 	if err != nil {
 		t.Fatal(err)
 	}
@@ -217,6 +218,14 @@ func TestClosingAndSocketCount(t *testing.T) {
 	}
 }
 
+func TestClosingAndSocketCountFastClient(t *testing.T) {
+	testClosingAndSocketCount(t, &HTTPRunnerOptions{})
+}
+
+func TestClosingAndSocketCountStdClient(t *testing.T) {
+	testClosingAndSocketCount(t, &HTTPRunnerOptions{HTTPOptions: HTTPOptions{DisableFastClient: true}})
+}
+
 func TestHTTPRunnerBadServer(t *testing.T) {
 	// Using http to an https server (or the current 'close all' dummy https server)
 	// should fail:
@@ -462,3 +471,113 @@ func TestAbortOn(t *testing.T) {
 		t.Errorf("Abort2 not working, did %d requests expecting ideally 1 and <= %d", count, o.NumThreads)
 	}
 }
+
+func TestConnectionReuseRange(t *testing.T) {
+	mux, addr := DynamicHTTPServer(false)
+	mux.HandleFunc("/foo/", EchoHandler)
+	url := fmt.Sprintf("http://localhost:%d/foo/", addr.Port)
+	opts := HTTPRunnerOptions{}
+	opts.Init(url)
+	opts.QPS = 50
+	opts.URL = url
+	opts.NumThreads = 1
+	opts.Exactly = 10
+
+	// Test empty reuse range.
+	res, err := RunHTTPTest(&opts)
+	if err != nil {
+		t.Error(err)
+	}
+
+	if res.SocketCount != 1 {
+		t.Errorf("Expected only 1 socket should be used when the max connection reuse flag is not set.")
+	}
+
+	// Test connection reuse range from 1 to 10.
+	for i := 1; i <= 10; i++ {
+		opts.ConnReuseRange = [2]int{i, i}
+		expectedSocketReuse := math.Ceil(float64(opts.Exactly) / float64(opts.ConnReuseRange[0]))
+		res, err := RunHTTPTest(&opts)
+		if err != nil {
+			t.Error(err)
+		}
+
+		if res.SocketCount != (int)(expectedSocketReuse) {
+			t.Errorf("Expecting %f socket to be used, got %d", expectedSocketReuse, res.SocketCount)
+		}
+	}
+
+	// Test when connection reuse range min != max.
+	// The actual socket count should always be 2 as the connection reuse range varies between 5 and 9.
+	expectedSocketReuse := 2
+	opts.ConnReuseRange = [2]int{5, 9}
+	// Check a few times that despite the range and random 2-9 we still always get 2 connections
+	for i := 0; i < 5; i++ {
+		res, err := RunHTTPTest(&opts)
+		if err != nil {
+			t.Error(err)
+		}
+
+		if res.SocketCount != expectedSocketReuse {
+			t.Errorf("Expecting %d socket to be used, got %d", expectedSocketReuse, res.SocketCount)
+		}
+	}
+}
+
+func TestValidateConnectionReuse(t *testing.T) {
+	httpOpts := HTTPOptions{}
+
+	err := httpOpts.ValidateAndSetConnectionReuseRange("1:2:3:4")
+	if err == nil {
+		t.Errorf("Shoud fail when more than two values are provided for connection reuse range.")
+	}
+
+	err = httpOpts.ValidateAndSetConnectionReuseRange("foo")
+	if err == nil {
+		t.Errorf("Shoud fail when non integer value is provided for connection reuse range.")
+	}
+
+	err = httpOpts.ValidateAndSetConnectionReuseRange("")
+	if err != nil {
+		t.Errorf("Expect no error when no value is privided, got err: %v.", err)
+	}
+
+	if httpOpts.ConnReuseRange != [2]int{0, 0} {
+		t.Errorf("Expect the connection reuse range to be [0, 0], got : %v.", httpOpts.ConnReuseRange)
+	}
+
+	err = httpOpts.ValidateAndSetConnectionReuseRange("10")
+	if err != nil {
+		t.Errorf("Expect no error when single value is privided, got err: %v.", err)
+	}
+
+	if httpOpts.ConnReuseRange != [2]int{10, 10} {
+		t.Errorf("Expect the connection reuse range to be [10, 10], got : %v.", httpOpts.ConnReuseRange)
+	}
+
+	err = httpOpts.ValidateAndSetConnectionReuseRange("20:10")
+	if err != nil {
+		t.Errorf("Expect no error when two values are privided, got err: %v.", err)
+	}
+
+	if httpOpts.ConnReuseRange != [2]int{10, 20} {
+		t.Errorf("Expect the connection reuse range to be [10, 20], got : %v.", httpOpts.ConnReuseRange)
+	}
+
+	err = httpOpts.ValidateAndSetConnectionReuseRange("10:20")
+	if err != nil {
+		t.Errorf("Expect no error when two values are privided, got err: %v", err)
+	}
+
+	if httpOpts.ConnReuseRange != [2]int{10, 20} {
+		t.Errorf("Expect the connection reuse range to be [10, 20], got : %v.", httpOpts.ConnReuseRange)
+	}
+}
+
+func getIPUsageCount(ipCountMap map[string]int) (count int) {
+	for _, v := range ipCountMap {
+		count += v
+	}
+
+	return count
+}
