@@ -110,7 +110,7 @@ var (
 	profileFlag     = flag.String("profile", "", "write .cpu and .mem profiles to `file`")
 	grpcFlag        = flag.Bool("grpc", false, "Use GRPC (health check by default, add -ping for ping) for load testing")
 	echoPortFlag    = flag.String("http-port", "8080",
-		"http echo server port. Can be in the form of host:port, ip:port, `port` or /unix/domain/path.")
+		"http echo server port. Can be in the form of host:port, ip:port, `port` or /unix/domain/path or \""+disabled+"\".")
 	tcpPortFlag = flag.String("tcp-port", "8078",
 		"tcp echo server port. Can be in the form of host:port, ip:port, `port` or /unix/domain/path or \""+disabled+"\".")
 	udpPortFlag = flag.String("udp-port", "8078",
@@ -130,15 +130,13 @@ var (
 	// do not remove the flag for backward compatibility.  Was absolute `path` to the dir containing the static files dir
 	// which is now embedded in the binary thanks to that support in golang 1.16.
 	_            = flag.String("static-dir", "", "Deprecated/unused `path`.")
-	dataDirFlag  = flag.String("data-dir", defaultDataDir, "`Directory` where JSON results are stored/read")
+	dataDirFlag  = flag.String("data-dir", ".", "`Directory` where JSON results are stored/read")
 	proxiesFlags proxiesFlagList
 	proxies      = make([]string, 0)
 	// -M flag.
 	httpMultiFlags httpMultiFlagList
 	httpMulties    = make([]string, 0)
 
-	defaultDataDir = "."
-
 	allowInitialErrorsFlag = flag.Bool("allow-initial-errors", false, "Allow and don't abort on initial warmup errors")
 	abortOnFlag            = flag.Int("abort-on", 0,
 		"Http `code` that if encountered aborts the run. e.g. 503 or -1 for socket errors.")
@@ -185,7 +183,7 @@ var (
 	calcQPS = flag.Bool("calc-qps", false, "Calculate the qps based on number of requests (-n) and duration (-t)")
 )
 
-// nolint: funlen // well yes it's fairly big and lotsa ifs.
+// nolint: funlen,gocyclo // well yes it's fairly big and lotsa ifs.
 func main() {
 	flag.Var(&proxiesFlags, "P",
 		"Tcp proxies to run, e.g -P \"localport1 dest_host1:dest_port1\" -P \"[::1]:0 www.google.com:443\" ...")
@@ -271,8 +269,10 @@ func main() {
 		if *redirectFlag != disabled {
 			fhttp.RedirectToHTTPS(*redirectFlag)
 		}
-		if !ui.Serve(baseURL, *echoPortFlag, *echoDbgPathFlag, *uiPathFlag, *dataDirFlag, percList) {
-			os.Exit(1) // error already logged
+		if *echoPortFlag != disabled {
+			if !ui.Serve(baseURL, *echoPortFlag, *echoDbgPathFlag, *uiPathFlag, *dataDirFlag, percList) {
+				os.Exit(1) // error already logged
+			}
 		}
 		startProxies()
 	case "grpcping":
@@ -366,7 +366,7 @@ func fortioLoad(justCurl bool, percList []float64) {
 		if *exactlyFlag == 0 || *durationFlag <= 0 {
 			usageErr("Error: can't use `-calc-qps` without also specifying `-n` and `-t`")
 		}
-		qps = float64(*exactlyFlag) / (*durationFlag).Seconds()
+		qps = float64(*exactlyFlag) / durationFlag.Seconds()
 		log.LogVf("Calculated QPS to do %d request in %v: %f", *exactlyFlag, *durationFlag, qps)
 	}
 	_, _ = fmt.Fprintf(out, "Fortio %s running at %g queries per second, %d->%d procs",
@@ -426,7 +426,7 @@ func fortioLoad(justCurl bool, percList []float64) {
 			Service:            *healthSvcFlag,
 			Streams:            *streamsFlag,
 			AllowInitialErrors: *allowInitialErrorsFlag,
-			Payload:            httpOpts.PayloadString(),
+			Payload:            httpOpts.PayloadUTF8(),
 			Delay:              *pingDelayFlag,
 			UsePing:            *doPingLoadFlag,
 		}
@@ -516,12 +516,19 @@ func grpcClient() {
 		count = 1
 	}
 	httpOpts := bincommon.SharedHTTPOptions()
-	var err error
 	if *doHealthFlag {
-		_, err = fgrpc.GrpcHealthCheck(host, *healthSvcFlag, count, &httpOpts.TLSOptions)
-	} else {
-		_, err = fgrpc.PingClientCall(host, count, httpOpts.PayloadString(), *pingDelayFlag, &httpOpts.TLSOptions)
+		status, err := fgrpc.GrpcHealthCheck(host, *healthSvcFlag, count, &httpOpts.TLSOptions)
+		if err != nil {
+			// already logged
+			os.Exit(1)
+		}
+		if (*status)["SERVING"] != int64(count) {
+			log.Errf("Unexpected SERVING count %d vs %d", (*status)["SERVING"], count)
+			os.Exit(1)
+		}
+		return
 	}
+	_, err := fgrpc.PingClientCall(host, count, httpOpts.PayloadUTF8(), *pingDelayFlag, &httpOpts.TLSOptions)
 	if err != nil {
 		// already logged
 		os.Exit(1)
