@@ -16,11 +16,8 @@
 package ui // import "fortio.org/fortio/ui"
 
 import (
-	"bytes"
 	"context"
-	"crypto/md5" // nolint: gosec // md5 is mandated, not our choice
 	"embed"
-	"encoding/base64"
 	"encoding/json"
 	"encoding/xml"
 	"flag"
@@ -35,15 +32,16 @@ import (
 	"path"
 	"strconv"
 	"strings"
-	"sync"
 	"time"
 
+	"fortio.org/fortio/bincommon"
 	"fortio.org/fortio/dflag/endpoint"
 	"fortio.org/fortio/fgrpc"
 	"fortio.org/fortio/fhttp"
 	"fortio.org/fortio/fnet"
 	"fortio.org/fortio/log"
 	"fortio.org/fortio/periodic"
+	"fortio.org/fortio/rapi"
 	"fortio.org/fortio/stats"
 	"fortio.org/fortio/tcprunner"
 	"fortio.org/fortio/udprunner"
@@ -73,28 +71,17 @@ var (
 	// Start time of the UI Server (for uptime info).
 	startTime        time.Time
 	extraBrowseLabel string // Extra label for report only
-	// Directory where results are written to/read from.
-	dataDir        string
-	mainTemplate   *template.Template
-	browseTemplate *template.Template
-	syncTemplate   *template.Template
-	uiRunMapMutex  = &sync.Mutex{}
-	id             int64
-	runs           = make(map[int64]*periodic.RunnerOptions)
+	mainTemplate     *template.Template
+	browseTemplate   *template.Template
+	syncTemplate     *template.Template
 	// Base URL used for index - useful when running under an ingress with prefix.
 	baseURL string
-
-	defaultPercentileList []float64
 )
 
 const (
-	fetchURI      = "fetch/"
-	fetch2URI     = "fetch2/"
-	restRunURI    = "rest/run"
-	restStatusURI = "rest/status"
-	restStopURI   = "rest/stop"
-	faviconPath   = "/favicon.ico"
-	modegrpc      = "grpc"
+	fetchURI    = "fetch/"
+	fetch2URI   = "fetch2/"
+	faviconPath = "/favicon.ico"
 )
 
 // TODO: auto map from (Http)RunnerOptions to form generation and/or accept
@@ -132,12 +119,10 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 		} else {
 			log.Infof("Starting %s load request from %v for %s", runner, r.RemoteAddr, url)
 		}
-	} else {
-		if r.FormValue("stop") == "Stop" {
-			runid, _ = strconv.ParseInt(r.FormValue("runid"), 10, 64)
-			log.Critf("Stop request from %v for %d", r.RemoteAddr, runid)
-			mode = stop
-		}
+	} else if r.FormValue("stop") == "Stop" {
+		runid, _ = strconv.ParseInt(r.FormValue("runid"), 10, 64)
+		log.Critf("Stop request from %v for %d", r.RemoteAddr, runid)
+		mode = stop
 	}
 	// Those only exist/make sense on run mode but go variable declaration...
 	payload := r.FormValue("payload")
@@ -146,6 +131,10 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	percList, _ := stats.ParsePercentiles(r.FormValue("p"))
 	qps, _ := strconv.ParseFloat(r.FormValue("qps"), 64)
 	durStr := r.FormValue("t")
+	connectionReuseRange := parseConnectionReuseRange(
+		r.FormValue("connection-reuse-range-min"),
+		r.FormValue("connection-reuse-range-max"),
+		r.FormValue("connection-reuse-range-value"))
 	jitter := (r.FormValue("jitter") == "on")
 	uniform := (r.FormValue("uniform") == "on")
 	nocatchup := (r.FormValue("nocatchup") == "on")
@@ -175,7 +164,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	}
 	out := io.Writer(os.Stderr)
 	if len(percList) == 0 && !strings.Contains(r.URL.RawQuery, "p=") {
-		percList = defaultPercentileList
+		percList = rapi.DefaultPercentileList
 	}
 	if !JSONOnly {
 		out = fhttp.NewHTMLEscapeWriter(w)
@@ -199,13 +188,9 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	}
 	if mode == run {
 		ro.Normalize()
-		uiRunMapMutex.Lock()
-		id++ // start at 1 as 0 means interrupt all
-		runid = id
-		runs[runid] = &ro
-		uiRunMapMutex.Unlock()
+		runid = rapi.AddRun(&ro)
 		log.Infof("New run id %d", runid)
-		ro.RunID = id
+		ro.RunID = runid
 	}
 	httpopts := &fhttp.HTTPOptions{}
 	httpopts.HTTPReqTimeOut = timeout // to be normalized in init 0 replaced by default value
@@ -216,6 +201,14 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	httpopts.SequentialWarmup = sequentialWarmup
 	httpopts.Insecure = httpsInsecure
 	httpopts.Resolve = resolve
+	// Set the connection reuse range.
+	err := bincommon.ConnectionReuseRange.
+		WithValidator(bincommon.ConnectionReuseRangeValidator(httpopts)).
+		Set(connectionReuseRange)
+	if err != nil {
+		log.Errf("Fail to validate connection reuse range flag, err: %v", err)
+	}
+
 	if len(payload) > 0 {
 		httpopts.Payload = []byte(payload)
 	}
@@ -266,7 +259,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 	case menu:
 		// nothing more to do
 	case stop:
-		StopByRunID(runid)
+		rapi.StopByRunID(runid)
 	case run:
 		// mode == run case:
 		for _, header := range r.Form["H"] {
@@ -285,7 +278,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 		}
 		var res periodic.HasRunnerResult
 		var err error
-		if runner == modegrpc {
+		if runner == rapi.ModeGRPC {
 			o := fgrpc.GRPCRunnerOptions{
 				RunnerOptions: ro,
 				Destination:   url,
@@ -324,9 +317,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 			}
 			res, err = fhttp.RunHTTPTest(&o)
 		}
-		uiRunMapMutex.Lock()
-		delete(runs, ro.RunID)
-		uiRunMapMutex.Unlock()
+		rapi.RemoveRun(ro.RunID)
 		if err != nil {
 			log.Errf("Init error for %s mode with url %s and options %+v : %v", runner, url, ro, err)
 
@@ -342,7 +333,7 @@ func Handler(w http.ResponseWriter, r *http.Request) {
 		savedAs := ""
 		id := res.Result().ID()
 		if doSave {
-			savedAs = SaveJSON(id, json)
+			savedAs = rapi.SaveJSON(id, json)
 		}
 		if JSONOnly {
 			w.Header().Set("Content-Type", "application/json")
@@ -373,23 +364,6 @@ func ResultToJsData(w io.Writer, json []byte) {
 	_, _ = w.Write([]byte("\nvar data = fortioResultToJsChartData(res)\nshowChart(data)\n"))
 }
 
-// SaveJSON save Json bytes to give file name (.json) in data-path dir.
-func SaveJSON(name string, json []byte) string {
-	if dataDir == "" {
-		log.Infof("Not saving because data-path is unset")
-		return ""
-	}
-	name += ".json"
-	log.Infof("Saving %s in %s", name, dataDir)
-	err := ioutil.WriteFile(path.Join(dataDir, name), json, 0o644) // nolint: gosec // we do want 644
-	if err != nil {
-		log.Errf("Unable to save %s in %s: %v", name, dataDir, err)
-		return ""
-	}
-	// Return the relative path from the /fortio/ UI
-	return "data/" + name
-}
-
 // SelectableValue represets an entry in the <select> of results.
 type SelectableValue struct {
 	Value    string
@@ -417,27 +391,6 @@ func SelectValues(values []string, selectedValues []string) (selectableValues []
 	return selectableValues, numSelected
 }
 
-// DataList returns the .json files/entries in data dir.
-func DataList() (dataList []string) {
-	files, err := ioutil.ReadDir(dataDir)
-	if err != nil {
-		log.Critf("Can list directory %s: %v", dataDir, err)
-		return
-	}
-	// Newest files at the top:
-	for i := len(files) - 1; i >= 0; i-- {
-		name := files[i].Name()
-		ext := ".json"
-		if !strings.HasSuffix(name, ext) || files[i].IsDir() {
-			log.LogVf("Skipping non %s file: %s", ext, name)
-			continue
-		}
-		dataList = append(dataList, name[:len(name)-len(ext)])
-	}
-	log.LogVf("data list is %v (out of %d files in %s)", dataList, len(files), dataDir)
-	return dataList
-}
-
 // ChartOptions describes the user-configurable options for a chart.
 type ChartOptions struct {
 	XMin   string
@@ -471,7 +424,7 @@ func BrowseHandler(w http.ResponseWriter, r *http.Request) {
 	yMin := r.FormValue("yMin")
 	yMax := r.FormValue("yMax")
 	yLog, _ := strconv.ParseBool(r.FormValue("yLog"))
-	dataList := DataList()
+	dataList := rapi.DataList()
 	selectedValues := r.URL.Query()["sel"]
 	preselectedDataList, numSelected := SelectValues(dataList, selectedValues)
 
@@ -529,7 +482,7 @@ func LogAndAddCacheControl(h http.Handler) http.Handler {
 func sendHTMLDataIndex(w http.ResponseWriter) {
 	w.Header().Set("Content-Type", "text/html; charset=UTF-8")
 	_, _ = w.Write([]byte("<html><body><ul>\n"))
-	for _, e := range DataList() {
+	for _, e := range rapi.DataList() {
 		_, _ = w.Write([]byte("<li><a href=\""))
 		_, _ = w.Write([]byte(e))
 		_, _ = w.Write([]byte(".json\">"))
@@ -539,67 +492,6 @@ func sendHTMLDataIndex(w http.ResponseWriter) {
 	_, _ = w.Write([]byte("</ul></body></html>"))
 }
 
-type tsvCache struct {
-	cachedDirTime time.Time
-	cachedResult  []byte
-}
-
-var (
-	gTSVCache      tsvCache
-	gTSVCacheMutex = &sync.Mutex{}
-)
-
-// format for gcloud transfer
-// https://cloud.google.com/storage/transfer/create-url-list
-func sendTSVDataIndex(urlPrefix string, w http.ResponseWriter) {
-	info, err := os.Stat(dataDir)
-	if err != nil {
-		log.Errf("Unable to stat %s: %v", dataDir, err)
-		w.WriteHeader(http.StatusServiceUnavailable)
-		return
-	}
-	gTSVCacheMutex.Lock() // Kind of a long time to hold a lock... hopefully the FS doesn't hang...
-	useCache := (info.ModTime() == gTSVCache.cachedDirTime) && (len(gTSVCache.cachedResult) > 0)
-	if !useCache {
-		var b bytes.Buffer
-		b.Write([]byte("TsvHttpData-1.0\n"))
-		for _, e := range DataList() {
-			fname := e + ".json"
-			f, err := os.Open(path.Join(dataDir, fname))
-			if err != nil {
-				log.Errf("Open error for %s: %v", fname, err)
-				continue
-			}
-			// nolint: gosec // This isn't a crypto hash, more like a checksum - and mandated by the spec above, not our choice
-			h := md5.New()
-			var sz int64
-			if sz, err = io.Copy(h, f); err != nil {
-				f.Close()
-				log.Errf("Copy/read error for %s: %v", fname, err)
-				continue
-			}
-			b.Write([]byte(urlPrefix))
-			b.Write([]byte(fname))
-			b.Write([]byte("\t"))
-			b.Write([]byte(strconv.FormatInt(sz, 10)))
-			b.Write([]byte("\t"))
-			b.Write([]byte(base64.StdEncoding.EncodeToString(h.Sum(nil))))
-			b.Write([]byte("\n"))
-		}
-		gTSVCache.cachedDirTime = info.ModTime()
-		gTSVCache.cachedResult = b.Bytes()
-	}
-	result := gTSVCache.cachedResult
-	lastModified := gTSVCache.cachedDirTime.Format(http.TimeFormat)
-	gTSVCacheMutex.Unlock()
-	log.Infof("Used cached %v to serve %d bytes TSV", useCache, len(result))
-	w.Header().Set("Content-Type", "text/plain; charset=UTF-8")
-	// Cloud transfer requires ETag
-	w.Header().Set("ETag", fmt.Sprintf("\"%s\"", lastModified))
-	w.Header().Set("Last-Modified", lastModified)
-	_, _ = w.Write(result)
-}
-
 // LogAndFilterDataRequest logs the data request.
 func LogAndFilterDataRequest(h http.Handler) http.Handler {
 	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
@@ -625,7 +517,7 @@ func LogAndFilterDataRequest(h http.Handler) http.Handler {
 				urlPrefix += uiPath + "data/" // base has been cleaned of trailing / in fortio_main
 			}
 			log.Infof("Prefix is '%s'", urlPrefix)
-			sendTSVDataIndex(urlPrefix, w)
+			rapi.SendTSVDataIndex(urlPrefix, w)
 			return
 		}
 		if !strings.HasSuffix(path, ".json") {
@@ -667,7 +559,7 @@ func (o outHTTPWriter) Flush() {
 
 // Sync is the non http equivalent of fortio/sync?url=u.
 func Sync(out io.Writer, u string, datadir string) bool {
-	dataDir = datadir
+	rapi.SetDataDir(datadir)
 	v := url.Values{}
 	v.Set("url", u)
 	// TODO: better context?
@@ -700,29 +592,31 @@ func SyncHandler(w http.ResponseWriter, r *http.Request) {
 	flusher.Flush()
 	o := fhttp.NewHTTPOptions(uStr)
 	fhttp.OnBehalfOf(o, r)
+	// Increase timeout:
+	o.HTTPReqTimeOut = 5 * time.Second
 	// If we had hundreds of thousands of entry we should stream, parallelize (connection pool)
 	// and not do multiple passes over the same data, but for small tsv this is fine.
-	// use std client to change the url and handle https:
+	// use std client to avoid chunked raw we can get with fast client:
 	client, _ := fhttp.NewStdClient(o)
 	if client == nil {
 		_, _ = w.Write([]byte("invalid url!<script>setPB(1,1)</script></body></html>\n"))
-		// too late to write headers
+		// too late to write headers for real case but we do it anyway for the Sync() startup case
+		w.WriteHeader(http.StatusServiceUnavailable)
 		return
 	}
 	code, data, _ := client.Fetch()
 	defer client.Close()
 	if code != http.StatusOK {
 		_, _ = w.Write([]byte(fmt.Sprintf("http error, code %d<script>setPB(1,1)</script></body></html>\n", code)))
-		// too late to write headers
+		// too late to write headers for real case but we do it anyway for the Sync() startup case
+		w.WriteHeader(code)
 		return
 	}
 	sdata := strings.TrimSpace(string(data))
 	if strings.HasPrefix(sdata, "TsvHttpData-1.0") {
 		processTSV(w, client, sdata)
-	} else {
-		if !processXML(w, client, data, uStr, 0) {
-			return
-		}
+	} else if !processXML(w, client, data, uStr, 0) {
+		return
 	}
 	_, _ = w.Write([]byte("</table>"))
 	_, _ = w.Write([]byte("\n</body></html>\n"))
@@ -845,7 +739,7 @@ func downloadOne(w http.ResponseWriter, client *fhttp.Client, name string, u str
 		_, _ = w.Write([]byte("<td>skipped (not json)"))
 		return
 	}
-	localPath := path.Join(dataDir, name)
+	localPath := path.Join(rapi.GetDataDir(), name)
 	_, err := os.Stat(localPath)
 	if err == nil {
 		_, _ = w.Write([]byte("<td>skipped (already exists)"))
@@ -894,7 +788,6 @@ func Serve(baseurl, port, debugpath, uipath, datadir string, percentileList []fl
 	}
 	fhttp.SetupPPROF(mux)
 	uiPath = uipath
-	dataDir = datadir
 	if uiPath[len(uiPath)-1] != '/' {
 		log.Warnf("Adding missing trailing / to UI path '%s'", uiPath)
 		uiPath += "/"
@@ -910,12 +803,8 @@ func Serve(baseurl, port, debugpath, uipath, datadir string, percentileList []fl
 	fhttp.CheckConnectionClosedHeader = true // needed for proxy to avoid errors
 
 	// New REST apis.
-	restRunPath := uiPath + restRunURI
-	mux.HandleFunc(restRunPath, RESTRunHandler)
-	restStatusPath := uiPath + restStatusURI
-	mux.HandleFunc(restStatusPath, RESTStatusHandler)
-	restStopPath := uiPath + restStopURI
-	mux.HandleFunc(restStopPath, RESTStopHandler)
+	rapi.AddHandlers(mux, uiPath, datadir)
+	rapi.DefaultPercentileList = percentileList
 
 	logoPath = version.Short() + "/static/img/fortio-logo-gradient-no-bg.svg"
 	chartJSPath = version.Short() + "/static/js/Chart.min.js"
@@ -951,31 +840,29 @@ func Serve(baseurl, port, debugpath, uipath, datadir string, percentileList []fl
 	mux.HandleFunc(uiPath+"flags", dflagEndPt.ListFlags)
 	mux.HandleFunc(dflagSetURL, dflagEndPt.SetFlag)
 
-	if dataDir != "" {
-		fs := http.FileServer(http.Dir(dataDir))
+	if datadir != "" {
+		fs := http.FileServer(http.Dir(datadir))
 		mux.Handle(uiPath+"data/", LogAndFilterDataRequest(http.StripPrefix(uiPath+"data", fs)))
 		if datadir == "." {
 			var err error
 			datadir, err = os.Getwd()
 			if err != nil {
 				log.Errf("Unable to get current directory: %v", err)
-				datadir = dataDir
 			}
 		}
-		fmt.Println("Data directory is", datadir)
+		log.Printf("Data directory is %s", datadir)
 	}
 	urlHostPort = fnet.NormalizeHostPort(port, addr)
-	uiMsg := "UI started - visit:\n"
+	uiMsg := "\t UI started - visit:\n\t\t"
 	if strings.Contains(urlHostPort, "-unix-socket=") {
 		uiMsg += fmt.Sprintf("fortio curl %s http://localhost%s", urlHostPort, uiPath)
 	} else {
 		uiMsg += fmt.Sprintf("http://%s%s", urlHostPort, uiPath)
 		if strings.Contains(urlHostPort, "localhost") {
-			uiMsg += "\n(or any host/ip reachable on this server)"
+			uiMsg += "\n\t (or any host/ip reachable on this server)"
 		}
 	}
 	fmt.Println(uiMsg)
-	defaultPercentileList = percentileList
 	return true
 }
 
@@ -997,8 +884,7 @@ func Report(baseurl, port, datadir string) bool {
 	}
 	fmt.Printf(uiMsg + "\n")
 	uiPath = "/"
-	dataDir = datadir
-	logoPath = version.Short() + "/static/img/logo.svg"
+	logoPath = version.Short() + "/static/img/fortio-logo-gradient-no-bg.svg"
 	chartJSPath = version.Short() + "/static/js/Chart.min.js"
 	fs := http.FileServer(http.FS(staticFS))
 	prefix := uiPath + version.Short()
@@ -1011,7 +897,18 @@ func Report(baseurl, port, datadir string) bool {
 	} else {
 		mux.HandleFunc(uiPath, BrowseHandler)
 	}
-	fsd := http.FileServer(http.Dir(dataDir))
+	rapi.SetDataDir(datadir) // needed for serving json and index.tsv
+	fsd := http.FileServer(http.Dir(datadir))
 	mux.Handle(uiPath+"data/", LogAndFilterDataRequest(http.StripPrefix(uiPath+"data", fsd)))
 	return true
 }
+
+func parseConnectionReuseRange(min string, max string, value string) string {
+	if min != "" && max != "" {
+		return fmt.Sprintf("%s:%s", min, max)
+	} else if value != "" {
+		return value
+	}
+
+	return ""
+}
